#name "Mine Sweeper v0.8"
/*
    Mine Sweeper v0.8 - a TI-85 Game
    Copyright (C) 1998  Kari Lavikka  <tuner@bdb.fi>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

/*
    You need the 'Small C' compiler in order to compile this program.
    It is available for download at: http://www.seashellinc.com/jdewey/ti/
    My homepage is available at: http://kellari.ml.org
*/

#include "usglib.h"
#include "func\pspr_nr.c"
#include "lib\hvline.c"
#include "lib\disp_dec.c"
#include "func\zcp.c"

#define BOARD_WIDTH 11
#define BOARD_HEIGHT 8
#define BOARD_LENGTH 88  /* 11x8 = 88 */
#define SPRITE_SIZE 8

#define MINES_RIDICULOUS 5
#define MINES_EASY 8
#define MINES_MEDIOCRE 12
#define MINES_HARD 16
#define MINES_IMPOSSIBLE 22

extern char sprite_block[], sprite_cursor[], 
            sprite_blank[], sprite_bomb[],
            sprite_marked[], titlepic[];

int cx, cy;                 /* coordinates of cursor */
int ocx, ocy;               /* old coordinates of cursor */
int enlarge_change;
int bombs_on_board;         /* how many bombs are there */
int bomb_counter;
int marked_blocks;
char *board[BOARD_LENGTH];  /* game board 11x8 */


main() {

   int key;
   
   cx = 1;
   cy = 1;
   ocx = 1;
   ocy = 1;
   marked_blocks = 0;

   bombs_on_board = intro();

   bomb_counter = BOARD_LENGTH;

   clrscr;
   decorations();

   init_board(bombs_on_board);
   draw_board();

   do {
      print_stats();
      if(bomb_counter <= bombs_on_board) life(); /* game over - you won */
      draw_block(ocx, ocy);
      draw_raw_block(sprite_cursor, cx, cy);
      key = read_keyboard();
      if(key == K_SECOND) {
         click_block();
      } else if (key == K_ALPHA) mark_block();
   } while (key != K_EXIT);

   quit();
}

sleep(n) int n; {
   /* sleep(n) - pauses for n/200ths of a second.
      delay() function was broken. */
#asm
   ld  hl,2
   add hl,sp
   ld  c,(hl)
   inc hl
   ld  b,(hl)
   #fncall delay
#endasm
}

intro() {
   int key;
   int bombs;

   load_zcp(titlepic, 0xFC00);

   key = getkey();

   if(key == K_F1) {
      bombs = MINES_RIDICULOUS;
   } else if(key == K_F2) {
      bombs = MINES_EASY;
   } else if(key == K_F3) {
      bombs = MINES_MEDIOCRE;
   } else if(key == K_F4) {
      bombs = MINES_HARD;
   } else if(key == K_F5) {
      bombs = MINES_IMPOSSIBLE;
   } else { quit(); }

   return bombs;
}

open_block(x, y) int x, y; {
   int temp;

   temp = board[bindex(x, y)];
   if(temp > 9) {
      if(temp < 20) bomb_counter--;
      temp = temp - 10;
      enlarge_change = 1;
      board[bindex(x, y)] = temp;
      draw_block(x, y);
   }
}

mark_block() {
   int temp;
   
   temp = board[bindex(cx, cy)];
   if(temp > 19) {
      temp = temp - 10;
      marked_blocks--;
   } else if(temp > 9) {
      temp = temp + 10;
      marked_blocks++;
   }
   board[bindex(cx, cy)] = temp;
   draw_block(cx, cy);
   sleep(35);
}

enlarge_hole() {
   int x, y;

   enlarge_change = 0;

   for(y = 1; y < BOARD_HEIGHT + 1; y++) {
      for(x = 1; x < BOARD_WIDTH + 1; x++) {
	   if(board[bindex(x, y)] == 0) {
            if(x > 1) {
               open_block(x - 1, y);
               if(y > 1) open_block(x - 1, y - 1);
               if(y < BOARD_HEIGHT) open_block(x - 1, y + 1);
            }
            if(x < BOARD_WIDTH) {
               open_block(x + 1, y);
               if(y > 1) open_block(x + 1, y - 1);
               if(y < BOARD_HEIGHT) open_block(x + 1, y + 1);
            }
            if(y > 1) open_block(x, y - 1);
            if(y < BOARD_HEIGHT) open_block(x, y + 1);
         }
      } 
   }

   return enlarge_change;
}         

read_keyboard() {
   int key;

   ocx = cx;
   ocy = cy;

   key = getkey();

   if (key == K_RIGHT) {
      cx++;
      if (cx > BOARD_WIDTH) cx = BOARD_WIDTH;
   } else if (key == K_LEFT) {
      cx--;
      if (cx < 1) cx = 1;
   } else if (key == K_UP) {
      cy++;
      if (cy > BOARD_HEIGHT) cy = BOARD_HEIGHT;
   } else if (key == K_DOWN) {
      cy--;
      if (cy < 1) cy = 1;
   }

   return key;
}

click_block() {
   int temp;
   int temp2;

   draw_block(cx, cy);

   temp2 = bindex(cx, cy);
   temp = board[temp2];
   if(temp > 9) {
      if(temp > 19) {
         mark_block(cx, cy);
      } else {
         temp = temp - 10;
         board[temp2] = temp;
         draw_block(cx, cy);
         if(temp > 0) sleep(27);
         if(temp == 9) death();
         if(temp < 9) bomb_counter--;
         while(enlarge_hole());
      }
   }
}

init_board(bombs) int bombs; {
/*
   0   = empty
   1-8 = number
   9   = bomb
   >9  = box has a block on it. Decrease ten to find out
         what's under it.
   >19 = user has marked the box to be a bomb. decrease 20
*/

   int count;
   int temp;
   int x, y;

   /* clear the board */
   for(count = 0; count < BOARD_LENGTH; count++) board[count] = 0;

   /* randomize some bombs */
   for(count = 0; count < bombs; count++) {
      do {
         temp = rand() * BOARD_LENGTH / 256;
      } while(board[temp] == 9);
      board[temp] = 9;
   }

   /* count bombs around a box */
   for(y=1;y<BOARD_HEIGHT+1;y++) {
      for(x=1;x<BOARD_WIDTH+1;x++) {
         board[bindex(x, y)] = count_bombs(x, y);
      }
   }

   /* overlay all the boxes with blocks */
   for(x=0;x<BOARD_LENGTH;x++) board[x] = board[x] + 10;
}

count_bombs(x, y) int x, y; {

   /* this function is way too big! eats ~1KB of machine code */

   int bombs;

#asm
; COUNT_BOMBS ALKAA ----------
#endasm

   bombs = 0;

   if(board[bindex(x, y)] == 9) return 9;

   if(x > 1) {
      if(board[bindex(x - 1, y)] == 9) bombs++;
      if(y > 1) if(board[bindex(x - 1, y - 1)] == 9) bombs++;
      if(y < BOARD_WIDTH) if(board[bindex(x - 1, y + 1)] == 9) bombs++;
   }
   if(x < BOARD_WIDTH) {
      if(board[bindex(x + 1, y)] == 9) bombs++;
      if(y > 1) if(board[bindex(x + 1, y - 1)] == 9) bombs++;
      if(y < BOARD_HEIGHT) if(board[bindex(x + 1, y + 1)] == 9) bombs++;
   }
      
   if(y > 1) if(board[bindex(x, y - 1)] == 9) bombs++;
   if(y < BOARD_HEIGHT) if(board[bindex(x, y + 1)] == 9) bombs++;

   return bombs;
#asm
; COUNT_BOMBS LOPPUU ---------
#endasm
}

draw_raw_block(sprite, x, y) int *sprite, x, y; {
   sprite_nr(sprite, (x - 1) * SPRITE_SIZE, (BOARD_HEIGHT - y) * SPRITE_SIZE);
}

quit() {
   _exit();
}

death() {
   setxy(4, 3);
   puts("KABOOOM!!!");
   getkey();
   quit();
}

life() {
   setxy(4, 2);
   puts("YOU DID IT!!");
   getkey();
   quit();
}


draw_block(x, y) int x, y; {
   int temp;   

   temp = bindex(x, y);
   if(board[temp] == 0) {
      draw_raw_block(sprite_blank, x, y);
   } else if(board[temp] == 9) {
      draw_raw_block(sprite_bomb, x, y);
   } else if(board[temp] > 19) {
      draw_raw_block(sprite_marked, x, y);
   } else if(board[temp] > 9) {
      draw_raw_block(sprite_block, x, y);
   } else {
      draw_raw_block(sprite_blank, x, y);
      setxy_m((x * SPRITE_SIZE) - 13, (BOARD_HEIGHT - y) * SPRITE_SIZE + 1);
      disp_char(board[temp]);
   }
}

bindex(x, y) char x, y; {
   y--;
   x--;
   return(y * BOARD_WIDTH + x);
}

print_stats() {
   setxy_m(119, 44);
   puts_m("   ");
   setxy_m(115, 44);
   disp_char(bomb_counter - bombs_on_board);
   setxy_m(119, 27);
   puts_m("   ");
   setxy_m(115, 27);
   disp_char(marked_blocks);
}

decorations() {
   vline(0, 63, 127);
   vline(0, 63, 89);
   hline(90, 126, 0);
   hline(90, 126, 63);
   setxy_m(91, 1);
   puts_m("Mine");
   setxy_m(97, 7);
   puts_m("Sweeper");
   hline(90, 126, 14);

   setxy_m(91, 17);
   puts_m("Mines:");
   setxy_m(115, 17);
   disp_char(bombs_on_board);

   setxy_m(91, 27);
   puts_m("Marked:");

   setxy_m(91, 37);
   puts_m("Blocks");
   setxy_m(91, 44);
   puts_m("left:");

   hline(90, 126, 55);
   setxy_m(91, 56);
   puts_m("(C)  KL     '98");
}

draw_board() {
   int x, y;
   for(y=1; y<BOARD_HEIGHT+1; y++) {
      for(x=1; x<BOARD_WIDTH+1; x++) {
         draw_block(x, y);
      }
   }
}

#asm

;GCP by Andreas Ess, ZCP(by SJ) compressed picture, size = 504 bytes
titlepic:
 .db $00,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F
 .db $3F,$3F,$3F,$3F,$3F,$3F,$1B,$F8,$C1,$F2,$8A,$08,$C0,$3F,$16,$C1
 .db $C1,$88,$88,$08,$C0,$3F,$16,$F0,$C1,$8A,$BA,$B5,$A6,$A9,$C0,$3F
 .db $09,$C0,$C1,$F2,$CA,$C5,$A9,$AA,$3F,$10,$C0,$C1,$8A,$CA,$C5,$A9
 .db $A8,$C0,$1F,$F8,$22,$C0,$C1,$8A,$BA,$B3,$A6,$9B,$26,$F8,$3F,$3A
 .db $F8,$22,$F9,$C1,$F0,$3F,$2C,$C2,$A1,$38,$FB,$EF,$CF,$BD,$F0,$BF
 .db $10,$F0,$A1,$E6,$9A,$C0,$23,$50,$BD,$49,$9F,$FC,$0C,$C1,$C1,$81
 .db $A2,$C0,$23,$50,$BD,$49,$9F,$FC,$0C,$C2,$81,$85,$89,$C0,$23,$FD
 .db $FB,$EF,$BE,$FB,$E1,$F0,$09,$C3,$E1,$F3,$B0,$C0,$23,$F9,$F3,$EF
 .db $BC,$FB,$4A,$27,$E0,$25,$F9,$F3,$EF,$BC,$FB,$4A,$0D,$F9,$C1,$C6
 .db $80,$D0,$23,$F9,$F3,$EF,$BC,$FB,$E0,$10,$C2,$A1,$C6,$80,$C0,$23
 .db $F9,$F3,$EF,$BC,$FB,$F1,$E0,$09,$F0,$C1,$AA,$A3,$D3,$8D,$C8,$15
 .db $F9,$F3,$EF,$BC,$F9,$49,$0E,$C0,$A1,$AA,$D4,$D4,$D1,$94,$15,$F9
 .db $F3,$EF,$BC,$F9,$49,$0E,$C2,$A1,$92,$C4,$D4,$D1,$90,$15,$F9,$F3
 .db $EF,$BC,$F8,$BF,$10,$C1,$C1,$92,$B3,$D3,$8D,$8C,$3F,$3F,$18,$F8
 .db $32,$F8,$C1,$90,$80,$C0,$23,$C2,$33,$C1,$C1,$90,$80,$C0,$23,$C0
 .db $C4,$A7,$8E,$AC,$B9,$A0,$09,$F1,$C1,$F6,$B3,$C0,$24,$E1,$94,$D1
 .db $A2,$E5,$8B,$0F,$C2,$C1,$91,$A4,$C0,$26,$E2,$D4,$FD,$FA,$97,$E8
 .db $0D,$C3,$E1,$95,$A4,$C0,$28,$CA,$D2,$84,$88,$D0,$A0,$0B,$C0,$C1
 .db $93,$A3,$C0,$23,$C2,$91,$88,$D1,$B2,$C5,$3F,$18,$F8,$A2,$8E,$9C
 .db $D8,$F2,$0F,$FB,$E1,$19,$D0,$C0,$2E,$C0,$16,$C2,$81,$1B,$C2,$33
 .db $C0,$16,$F3,$C1,$BC,$F1,$C6,$B5,$E4,$C0,$3F,$09,$C0,$A1,$AA,$CA
 .db $A8,$C5,$95,$A0,$3F,$09,$C0,$A1,$AA,$CA,$A2,$95,$95,$3F,$10,$C3
 .db $C1,$AA,$F1,$CC,$E5,$E4,$E0,$3F,$1E,$C0,$3F,$3A,$C0,$3F,$3F,$3F
 .db $3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$3F,$30
 .db $7F,$7F,$E0,$3F,$3F,$CC,$C4,$C0,$92,$0A,$D1,$0B,$C8,$C4,$82,$14
 .db $E2,$85,$81,$A8,$92,$A4,$D0,$08,$C0,$80,$C4,$08,$EA,$D5,$82,$A0
 .db $0D,$C9,$82,$C1,$82,$89,$82,$B0,$ED,$90,$EA,$D5,$A6,$85,$EE,$A0
 .db $A6,$D6,$93,$AD,$B1,$D8,$B4,$C2,$A4,$CA,$AA,$A4,$AA,$D6,$B2,$C1
 .db $88,$D4,$89,$95,$AA,$CB,$EA,$D5,$89,$90,$C6,$92,$A6,$C9,$E6,$A5
 .db $AA,$B0,$A4,$A2,$82,$93,$AB,$91,$83,$8D,$CA,$A4,$88,$88,$3F,$A0
 .db $10,$E0,$11,$C0,$3F,$3E,$A0,$00

sprite_block:
.db 8, 8
.db %00000001
.db %00101011
.db %01010111
.db %00101011
.db %01010111
.db %00101011
.db %01111111
.db %11111111

sprite_marked:
.db 8, 8
.db %00000000
.db %01111110
.db %01111110
.db %01111110
.db %01111110
.db %01111110
.db %01111110
.db %00000000

sprite_cursor:
.db 8, 8
.db %00011000
.db %01111110
.db %01100110
.db %11000011
.db %11000011
.db %01100110
.db %01111110
.db %00011000

sprite_blank:
.db 8, 8
.db %00000000
.db %00000000
.db %00000000
.db %00000000
.db %00000000
.db %00000000
.db %00000000
.db %00000000

sprite_bomb:
.db 8, 8
.db %00000000
.db %00000110
.db %00001000
.db %00011100
.db %00111110
.db %00111110
.db %00011100
.db %00000000
#endasm